/*
 * Copyright (c) 2005,2015 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _HARMONICA_H
#define _HARMONICA_H

#include "driveParam.h"

/* Can Ids of various controller components */
typedef struct CAN_PARAM_OPEN_TYPE {
	enum CAN_OPEN_ID txPDO1;
	enum CAN_OPEN_ID txPDO2;
	enum CAN_OPEN_ID rxPDO2;
	enum CAN_OPEN_ID txSDO;
	enum CAN_OPEN_ID rxSDO;
} CAN_PARAM_OPEN_TYPE;

/* Motion type of the controller */
typedef enum CAN_MOTION_TYPE {
	MOTIONTYPE_VELCTRL,
	MOTIONTYPE_POSCTRL
} CAN_MOTION_TYPE;

/* Structure describing one particular motor controller */
typedef struct CAN_HARMONICA_STR {
	struct CAN_MOTORS_STR *parent;
	int dev;
	pthread_cond_t *started;

	CAN_PARAM_OPEN_TYPE canOpen;
	struct CAN_DRIVE_PARAMS driveParam;
	int divForRequestStatus;
	double canTimeout;

	CAN_MOTION_TYPE motionType;
	int statusCtrl;
	
	struct timespec *currentTime;
	struct timespec *watchdogTime;
	struct timespec *velCalcTime;
	struct timespec *failureStartTime;
	struct timespec *sendTime;

	int refVelGearIncS;	/* reference velocity */

	double angleGearRadMem;
	double velGearMeasRadS;
	double posGearMeasRad;

	bool limSwLeft;
	bool limSwRight;
	
	double oldPos;

	int motorState;
	int newMotorState;
	
	int countRequestDiv;

	bool currentLimitOn;
	int numAttempsRecFail;

} CAN_HARMONICA_STR;

extern CAN_HARMONICA_STR *harmonicaInit(int, CAN_MOTOR_ID);
extern void harmonicaEnd(CAN_HARMONICA_STR *);
extern int harmonicaStart(CAN_HARMONICA_STR *);
extern void harmonicaStop(CAN_HARMONICA_STR *);
extern void harmonicaRequestStatus(CAN_HARMONICA_STR *);
extern int harmonicaReset(CAN_HARMONICA_STR *);
extern void harmonicaInitCtrl(CAN_HARMONICA_STR *);
extern void harmonicaStartWatchdog(CAN_HARMONICA_STR *, bool);
extern void harmonicaSetGearVelRadS(CAN_HARMONICA_STR *, double);
extern bool harmonicaDecode(CAN_HARMONICA_STR *, CAN_MSG *);
extern void harmonicaShow(CAN_HARMONICA_STR *);

#endif
